@@ -24,6 +24,8 @@ from hive_metastore.ttypes import (
     AlreadyExistsException,
     FieldSchema,
     InvalidOperationException,
+    LockResponse,
+    LockState,
     MetaException,
     NoSuchObjectException,
     SerDeInfo,
@@ -34,12 +36,19 @@ from hive_metastore.ttypes import Database as HiveDatabase
 from hive_metastore.ttypes import Table as HiveTable
 
 from pyiceberg.catalog import PropertiesUpdateSummary
-from pyiceberg.catalog.hive import HiveCatalog, _construct_hive_storage_descriptor
+from pyiceberg.catalog.hive import (
+    LOCK_CHECK_MAX_WAIT_TIME,
+    LOCK_CHECK_MIN_WAIT_TIME,
+    LOCK_CHECK_RETRIES,
+    HiveCatalog,
+    _construct_hive_storage_descriptor,
+)
 from pyiceberg.exceptions import (
     NamespaceAlreadyExistsError,
     NamespaceNotEmptyError,
     NoSuchNamespaceError,
     NoSuchTableError,
+    WaitingForLockException,
 )
 from pyiceberg.partitioning import PartitionField, PartitionSpec
 from pyiceberg.schema import Schema
@@ -225,27 +234,27 @@ def test_create_table(
             retention=None,
             sd=StorageDescriptor(
                 cols=[
-                    FieldSchema(name='boolean', type='boolean', comment=None),
-                    FieldSchema(name='integer', type='int', comment=None),
-                    FieldSchema(name='long', type='bigint', comment=None),
-                    FieldSchema(name='float', type='float', comment=None),
-                    FieldSchema(name='double', type='double', comment=None),
-                    FieldSchema(name='decimal', type='decimal(32,3)', comment=None),
-                    FieldSchema(name='date', type='date', comment=None),
-                    FieldSchema(name='time', type='string', comment=None),
-                    FieldSchema(name='timestamp', type='timestamp', comment=None),
+                    FieldSchema(name="boolean", type="boolean", comment=None),
+                    FieldSchema(name="integer", type="int", comment=None),
+                    FieldSchema(name="long", type="bigint", comment=None),
+                    FieldSchema(name="float", type="float", comment=None),
+                    FieldSchema(name="double", type="double", comment=None),
+                    FieldSchema(name="decimal", type="decimal(32,3)", comment=None),
+                    FieldSchema(name="date", type="date", comment=None),
+                    FieldSchema(name="time", type="string", comment=None),
+                    FieldSchema(name="timestamp", type="timestamp", comment=None),
                     FieldSchema(
-                        name='timestamptz',
-                        type='timestamp' if hive2_compatible else 'timestamp with local time zone',
+                        name="timestamptz",
+                        type="timestamp" if hive2_compatible else "timestamp with local time zone",
                         comment=None,
                     ),
-                    FieldSchema(name='string', type='string', comment=None),
-                    FieldSchema(name='uuid', type='string', comment=None),
-                    FieldSchema(name='fixed', type='binary', comment=None),
-                    FieldSchema(name='binary', type='binary', comment=None),
-                    FieldSchema(name='list', type='array<string>', comment=None),
-                    FieldSchema(name='map', type='map<string,int>', comment=None),
-                    FieldSchema(name='struct', type='struct<inner_string:string,inner_int:int>', comment=None),
+                    FieldSchema(name="string", type="string", comment=None),
+                    FieldSchema(name="uuid", type="string", comment=None),
+                    FieldSchema(name="fixed", type="binary", comment=None),
+                    FieldSchema(name="binary", type="binary", comment=None),
+                    FieldSchema(name="list", type="array<string>", comment=None),
+                    FieldSchema(name="map", type="map<string,int>", comment=None),
+                    FieldSchema(name="struct", type="struct<inner_string:string,inner_int:int>", comment=None),
                 ],
                 location=f"{hive_database.locationUri}/table",
                 inputFormat="org.apache.hadoop.mapred.FileInputFormat",
@@ -305,50 +314,225 @@ def test_create_table(
         last_column_id=22,
         schemas=[
             Schema(
-                NestedField(field_id=1, name='boolean', field_type=BooleanType(), required=True),
-                NestedField(field_id=2, name='integer', field_type=IntegerType(), required=True),
-                NestedField(field_id=3, name='long', field_type=LongType(), required=True),
-                NestedField(field_id=4, name='float', field_type=FloatType(), required=True),
-                NestedField(field_id=5, name='double', field_type=DoubleType(), required=True),
-                NestedField(field_id=6, name='decimal', field_type=DecimalType(precision=32, scale=3), required=True),
-                NestedField(field_id=7, name='date', field_type=DateType(), required=True),
-                NestedField(field_id=8, name='time', field_type=TimeType(), required=True),
-                NestedField(field_id=9, name='timestamp', field_type=TimestampType(), required=True),
-                NestedField(field_id=10, name='timestamptz', field_type=TimestamptzType(), required=True),
-                NestedField(field_id=11, name='string', field_type=StringType(), required=True),
-                NestedField(field_id=12, name='uuid', field_type=UUIDType(), required=True),
-                NestedField(field_id=13, name='fixed', field_type=FixedType(length=12), required=True),
-                NestedField(field_id=14, name='binary', field_type=BinaryType(), required=True),
+                NestedField(field_id=1, name="boolean", field_type=BooleanType(), required=True),
+                NestedField(field_id=2, name="integer", field_type=IntegerType(), required=True),
+                NestedField(field_id=3, name="long", field_type=LongType(), required=True),
+                NestedField(field_id=4, name="float", field_type=FloatType(), required=True),
+                NestedField(field_id=5, name="double", field_type=DoubleType(), required=True),
+                NestedField(field_id=6, name="decimal", field_type=DecimalType(precision=32, scale=3), required=True),
+                NestedField(field_id=7, name="date", field_type=DateType(), required=True),
+                NestedField(field_id=8, name="time", field_type=TimeType(), required=True),
+                NestedField(field_id=9, name="timestamp", field_type=TimestampType(), required=True),
+                NestedField(field_id=10, name="timestamptz", field_type=TimestamptzType(), required=True),
+                NestedField(field_id=11, name="string", field_type=StringType(), required=True),
+                NestedField(field_id=12, name="uuid", field_type=UUIDType(), required=True),
+                NestedField(field_id=13, name="fixed", field_type=FixedType(length=12), required=True),
+                NestedField(field_id=14, name="binary", field_type=BinaryType(), required=True),
                 NestedField(
                     field_id=15,
-                    name='list',
-                    field_type=ListType(type='list', element_id=18, element_type=StringType(), element_required=True),
+                    name="list",
+                    field_type=ListType(type="list", element_id=18, element_type=StringType(), element_required=True),
                     required=True,
                 ),
                 NestedField(
                     field_id=16,
-                    name='map',
+                    name="map",
                     field_type=MapType(
-                        type='map', key_id=19, key_type=StringType(), value_id=20, value_type=IntegerType(), value_required=True
+                        type="map", key_id=19, key_type=StringType(), value_id=20, value_type=IntegerType(), value_required=True
                     ),
                     required=True,
                 ),
                 NestedField(
                     field_id=17,
-                    name='struct',
+                    name="struct",
                     field_type=StructType(
-                        NestedField(field_id=21, name='inner_string', field_type=StringType(), required=False),
-                        NestedField(field_id=22, name='inner_int', field_type=IntegerType(), required=True),
+                        NestedField(field_id=21, name="inner_string", field_type=StringType(), required=False),
+                        NestedField(field_id=22, name="inner_int", field_type=IntegerType(), required=True),
                     ),
-                    required=True,
+                    required=False,
                 ),
                 schema_id=0,
                 identifier_field_ids=[2],
             )
         ],
         current_schema_id=0,
-        last_partition_id=1000,
-        properties={"owner": "javaberg", 'write.parquet.compression-codec': 'zstd'},
+        last_partition_id=999,
+        properties={"owner": "javaberg", "write.parquet.compression-codec": "zstd"},
+        partition_specs=[PartitionSpec()],
+        default_spec_id=0,
+        current_snapshot_id=None,
+        snapshots=[],
+        snapshot_log=[],
+        metadata_log=[],
+        sort_orders=[SortOrder(order_id=0)],
+        default_sort_order_id=0,
+        refs={},
+        format_version=2,
+        last_sequence_number=0,
+    )
+
+    assert metadata.model_dump() == expected.model_dump()
+
+
+@pytest.mark.parametrize("hive2_compatible", [True, False])
+@patch("time.time", MagicMock(return_value=12345))
+def test_create_table_with_given_location_removes_trailing_slash(
+    table_schema_with_all_types: Schema, hive_database: HiveDatabase, hive_table: HiveTable, hive2_compatible: bool
+) -> None:
+    catalog = HiveCatalog(HIVE_CATALOG_NAME, uri=HIVE_METASTORE_FAKE_URL)
+    if hive2_compatible:
+        catalog = HiveCatalog(HIVE_CATALOG_NAME, uri=HIVE_METASTORE_FAKE_URL, **{"hive.hive2-compatible": "true"})
+
+    location = f"{hive_database.locationUri}/table-given-location"
+
+    catalog._client = MagicMock()
+    catalog._client.__enter__().create_table.return_value = None
+    catalog._client.__enter__().get_table.return_value = hive_table
+    catalog._client.__enter__().get_database.return_value = hive_database
+    catalog.create_table(
+        ("default", "table"), schema=table_schema_with_all_types, properties={"owner": "javaberg"}, location=f"{location}/"
+    )
+
+    called_hive_table: HiveTable = catalog._client.__enter__().create_table.call_args[0][0]
+    # This one is generated within the function itself, so we need to extract
+    # it to construct the assert_called_with
+    metadata_location: str = called_hive_table.parameters["metadata_location"]
+    assert metadata_location.endswith(".metadata.json")
+    assert "/database/table-given-location/metadata/" in metadata_location
+    catalog._client.__enter__().create_table.assert_called_with(
+        HiveTable(
+            tableName="table",
+            dbName="default",
+            owner="javaberg",
+            createTime=12345,
+            lastAccessTime=12345,
+            retention=None,
+            sd=StorageDescriptor(
+                cols=[
+                    FieldSchema(name="boolean", type="boolean", comment=None),
+                    FieldSchema(name="integer", type="int", comment=None),
+                    FieldSchema(name="long", type="bigint", comment=None),
+                    FieldSchema(name="float", type="float", comment=None),
+                    FieldSchema(name="double", type="double", comment=None),
+                    FieldSchema(name="decimal", type="decimal(32,3)", comment=None),
+                    FieldSchema(name="date", type="date", comment=None),
+                    FieldSchema(name="time", type="string", comment=None),
+                    FieldSchema(name="timestamp", type="timestamp", comment=None),
+                    FieldSchema(
+                        name="timestamptz",
+                        type="timestamp" if hive2_compatible else "timestamp with local time zone",
+                        comment=None,
+                    ),
+                    FieldSchema(name="string", type="string", comment=None),
+                    FieldSchema(name="uuid", type="string", comment=None),
+                    FieldSchema(name="fixed", type="binary", comment=None),
+                    FieldSchema(name="binary", type="binary", comment=None),
+                    FieldSchema(name="list", type="array<string>", comment=None),
+                    FieldSchema(name="map", type="map<string,int>", comment=None),
+                    FieldSchema(name="struct", type="struct<inner_string:string,inner_int:int>", comment=None),
+                ],
+                location=f"{hive_database.locationUri}/table-given-location",
+                inputFormat="org.apache.hadoop.mapred.FileInputFormat",
+                outputFormat="org.apache.hadoop.mapred.FileOutputFormat",
+                compressed=None,
+                numBuckets=None,
+                serdeInfo=SerDeInfo(
+                    name=None,
+                    serializationLib="org.apache.hadoop.hive.serde2.lazy.LazySimpleSerDe",
+                    parameters=None,
+                    description=None,
+                    serializerClass=None,
+                    deserializerClass=None,
+                    serdeType=None,
+                ),
+                bucketCols=None,
+                sortCols=None,
+                parameters=None,
+                skewedInfo=None,
+                storedAsSubDirectories=None,
+            ),
+            partitionKeys=None,
+            parameters={"EXTERNAL": "TRUE", "table_type": "ICEBERG", "metadata_location": metadata_location},
+            viewOriginalText=None,
+            viewExpandedText=None,
+            tableType="EXTERNAL_TABLE",
+            privileges=None,
+            temporary=False,
+            rewriteEnabled=None,
+            creationMetadata=None,
+            catName=None,
+            ownerType=1,
+            writeId=-1,
+            isStatsCompliant=None,
+            colStats=None,
+            accessType=None,
+            requiredReadCapabilities=None,
+            requiredWriteCapabilities=None,
+            id=None,
+            fileMetadata=None,
+            dictionary=None,
+            txnId=None,
+        )
+    )
+
+    with open(metadata_location, encoding=UTF8) as f:
+        payload = f.read()
+
+    metadata = TableMetadataUtil.parse_raw(payload)
+
+    assert "database/table-given-location" in metadata.location
+
+    expected = TableMetadataV2(
+        location=metadata.location,
+        table_uuid=metadata.table_uuid,
+        last_updated_ms=metadata.last_updated_ms,
+        last_column_id=22,
+        schemas=[
+            Schema(
+                NestedField(field_id=1, name="boolean", field_type=BooleanType(), required=True),
+                NestedField(field_id=2, name="integer", field_type=IntegerType(), required=True),
+                NestedField(field_id=3, name="long", field_type=LongType(), required=True),
+                NestedField(field_id=4, name="float", field_type=FloatType(), required=True),
+                NestedField(field_id=5, name="double", field_type=DoubleType(), required=True),
+                NestedField(field_id=6, name="decimal", field_type=DecimalType(precision=32, scale=3), required=True),
+                NestedField(field_id=7, name="date", field_type=DateType(), required=True),
+                NestedField(field_id=8, name="time", field_type=TimeType(), required=True),
+                NestedField(field_id=9, name="timestamp", field_type=TimestampType(), required=True),
+                NestedField(field_id=10, name="timestamptz", field_type=TimestamptzType(), required=True),
+                NestedField(field_id=11, name="string", field_type=StringType(), required=True),
+                NestedField(field_id=12, name="uuid", field_type=UUIDType(), required=True),
+                NestedField(field_id=13, name="fixed", field_type=FixedType(length=12), required=True),
+                NestedField(field_id=14, name="binary", field_type=BinaryType(), required=True),
+                NestedField(
+                    field_id=15,
+                    name="list",
+                    field_type=ListType(type="list", element_id=18, element_type=StringType(), element_required=True),
+                    required=True,
+                ),
+                NestedField(
+                    field_id=16,
+                    name="map",
+                    field_type=MapType(
+                        type="map", key_id=19, key_type=StringType(), value_id=20, value_type=IntegerType(), value_required=True
+                    ),
+                    required=True,
+                ),
+                NestedField(
+                    field_id=17,
+                    name="struct",
+                    field_type=StructType(
+                        NestedField(field_id=21, name="inner_string", field_type=StringType(), required=False),
+                        NestedField(field_id=22, name="inner_int", field_type=IntegerType(), required=True),
+                    ),
+                    required=False,
+                ),
+                schema_id=0,
+                identifier_field_ids=[2],
+            )
+        ],
+        current_schema_id=0,
+        last_partition_id=999,
+        properties={"owner": "javaberg", "write.parquet.compression-codec": "zstd"},
         partition_specs=[PartitionSpec()],
         default_spec_id=0,
         current_snapshot_id=None,
@@ -400,7 +584,7 @@ def test_create_v1_table(table_schema_simple: Schema, hive_database: HiveDatabas
         schema=expected_schema,
         schemas=[expected_schema],
         current_schema_id=0,
-        last_partition_id=1000,
+        last_partition_id=999,
         properties={"owner": "javaberg", "write.parquet.compression-codec": "zstd"},
         partition_spec=[],
         partition_specs=[expected_spec],
@@ -983,3 +1167,31 @@ def test_resolve_table_location_warehouse(hive_database: HiveDatabase) -> None:
 
     location = catalog._resolve_table_location(None, "database", "table")
     assert location == "/tmp/warehouse/database.db/table"
+
+
+def test_hive_wait_for_lock() -> None:
+    lockid = 12345
+    acquired = LockResponse(lockid=lockid, state=LockState.ACQUIRED)
+    waiting = LockResponse(lockid=lockid, state=LockState.WAITING)
+    prop = {
+        "uri": HIVE_METASTORE_FAKE_URL,
+        LOCK_CHECK_MIN_WAIT_TIME: 0.1,
+        LOCK_CHECK_MAX_WAIT_TIME: 0.5,
+        LOCK_CHECK_RETRIES: 5,
+    }
+    catalog = HiveCatalog(HIVE_CATALOG_NAME, **prop)  # type: ignore
+    catalog._client = MagicMock()
+    catalog._client.lock.return_value = LockResponse(lockid=lockid, state=LockState.WAITING)
+
+    # lock will be acquired after 3 retries
+    catalog._client.check_lock.side_effect = [waiting if i < 2 else acquired for i in range(10)]
+    response: LockResponse = catalog._wait_for_lock("db", "tbl", lockid, catalog._client)
+    assert response.state == LockState.ACQUIRED
+    assert catalog._client.check_lock.call_count == 3
+
+    # lock wait should exit with WaitingForLockException finally after enough retries
+    catalog._client.check_lock.side_effect = [waiting for _ in range(10)]
+    catalog._client.check_lock.call_count = 0
+    with pytest.raises(WaitingForLockException):
+        catalog._wait_for_lock("db", "tbl", lockid, catalog._client)
+    assert catalog._client.check_lock.call_count == 5
